// 升级包表
import { http } from "@/utils/http";
type upgradePackageTable = {
  tableId: number,
  tableVersion: string,
  tableBanner?: string,
  tableName: string,
  tableDescription: string,
  tableDownload: number,
  tableUpdateTime: Date,
  tableIsSetUp: boolean,
  tableMark: string,
  tableOS: string,
  tableOSVersion: string,
  tableCreator: string
}
type Result = {
  success: boolean;
  data?: {
    /** 列表数据 */
    list: Array<upgradePackageTable>;
    /** 信息 */
    msg?: string;
  };
};

type PackageResult = {
    success: boolean;
  data?: {
    /** 列表数据 */
    list: upgradePackageTable;
    /** 信息 */
    msg?: string;
  };
}
export type upgradePackageTableForm = {
  tableId: number,
  tableVersion: string,
  tableBanner?: string,
  tableName: string,
  tablDescription: string,
  tableIsSetUp: boolean,
  tableMark: string,
}
/** 获取升级包列表元素 */
export const getUpgradePackageTableList = (data?: object) => { 
  return http.request<Result>("post", "/get-upgrade-package-list", { data });
}

/** 删除升级包表元素 */
export const deleteUpgradePackageTable = (data?:{tableId: number}) => { 
  return http.request<Result>("post", "/delete-upgrade-package-table", { data });
}

/** 修改升级包表元素 */
export const alterUpgradePackageTable = (data?: upgradePackageTableForm) => {
return http.request<Result>("post", "/alter-upgrade-package-table", { data });
}
 
/** 添加升级包表元素 */
export const addUpgradePackageTable = (data?: upgradePackageTableForm) => {
return http.request<Result>("post", "/add-upgrade-package-table", { data });
}

/** 按照升级包表id查 */
export const searchUpgradePackageTableById = (data?: {tableId: number}) => { 
  return http.request<PackageResult>("post", "/search-upgrade-package-table-by-id", { data });
}

/** 按照升级包表id查升级包表说明 */
export const searchUpgradePackageTableDescriptionById = (data?: {tableId: number}) => { 
  return http.request<Result>("post", "/search-upgrade-package-table-description-by-id", { data });
}

/** 修改对应升级包表下的备注信息 */
export const alterUpgradePackageTableDescriptionById = (data?: {tableId: number,description: string}) => { 
  return http.request<Result>("post", "/alter-upgrade-package-table-description-by-id", { data });
}


type VersionList = {
  packageId: number,
  packageVersion: string,
  packageDownload: number,
  packageType: number,
  createTime: Date,
  ossPath: string,
  packageName: string;
}

type VersionListResult = {
    success: boolean;
  data?: {
    /** 列表数据 */
    list: Array<VersionList>;
    /** 信息 */
    msg?: string;
  };
}

/** 根据升级包表id得到对应升级包表下的所有版本 */
export const searchUpgradePackageTableVersionById = (data?: {tableId: number}) => { 
  return http.request<VersionListResult>("post", "/search-upgrade-package-table-version-by-id", { data });
}


type RequireData = {
  thisWeek: Array<number>,
  lastWeek: Array<number>,
}

type RequireDataResult = {
  success: boolean,
  data: RequireData,
}

/** 此方法用于获取对应升级包表下的上周和本周的下载书 */
export const getRequireData = (data?: { tableId: number }) =>{
  return http.request<RequireDataResult>("post", "/get-require-data", { data });
}

type DepolyPercentResult = {
  success: boolean,
  data:{
    value: string,
    percent: string,
  }
}


/** 此方法用于获取对应升级包表下的安装成功率 */
export const getDepolyPercent = (data?: { tableId: number }) => {
  return http.request<DepolyPercentResult>("post", "/get-depoly-percent", { data });
}