// 测试连接的功能
// 每次从远程数据库获取已添加的设备
// 获取后，对已添加的设备逐一检测联通性
import { getToken } from "@/utils/auth";
import { http } from "@/utils/http";
import { storageSession } from "@pureadmin/utils";
import axios from "axios";
import { connect } from "echarts";
import { config } from "localforage";
type Connect = {
  // 设备编号
  connectId: number,
  // ip地址
  address: string;
  // ip地址的描述
  description: string;
  // 是否可联通
  isReachable?: string | null;
  // 最后一次测试的时间
  lastTested?: Date | null;
  // 延迟
  ping?: string;
  // 设备名
  connectName: string;
}

type ConnectResult = {
  success: boolean,
  data?: {
    ping: string
  }
}

/** 基于测试ip */

/**
 * 此方法用于实现测试ip的联通性
 */
export const testConnect = (data:string) => {
  return axios.post<ConnectResult>('http://' + data + ':8080' + "/api/ping", {},{
    headers: {
      "Authentication": getToken().accessToken
    }
  }
  );
}




type ConnectSearchResult = {
  success: boolean;
  data: {
    list: Array<Connect>,
    customeList: Array<Connect>,
    total: number,
    pageSize: number,
    currentPage: number,
  }

}

type DefaultResult = {
  success: boolean;
  data?:object
}


/**基于云端后端 */

const dottedDecimalToCIDR = (mask: string): number => {  
      const octets = mask.split('.').map(Number);  
      if (octets.length !== 4 || octets.some(octet => octet < 0 || octet > 255)) {  
        throw new Error('Invalid subnet mask');  
      }  

      let binaryString = octets  
        .map(octet => {  
          const binary = octet.toString(2);  
          return '0'.repeat(8 - binary.length) + binary;  
        })  
        .join('');  

      return (binaryString.match(/1/g) || []).length;  
};  


const calculateSubnets = (ip: string, mask: string): string[] => {  

  let ipList = ip.split('.').map(Number);  
  let maskList = mask.split('.').map(Number);
  let binaryList = [];
  for (let i = 0; i < 4; i++){
    const binary = ipList.at(i) & maskList.at(i)
    binaryList.push(binary);
  }
  console.log(binaryList);
  const resultList = [];
  const result = []
  const pow = 32 - dottedDecimalToCIDR(mask);
  for (let i = 0; i < Math.pow(2, pow); i++){
    if (i > Math.pow(2,8) - 1 && i <= Math.pow(2,16)-1) {
      const go = parseInt(i / Math.pow(2, 8));
      result[3] = i - go * Math.pow(2, 8) + binaryList[3];
      result[2] = go + binaryList[2];
      resultList.push(binaryList[0] + "." + binaryList[1] + "." + result[2] + "." + result[3]);
      continue
    }
    if (i > Math.pow(2,16) - 1 && i <=  Math.pow(2, 24) - 1) {
      const gogo = parseInt(i / Math.pow(2, 16));
      const go = parseInt(i - gogo * Math.pow(2, 16) / Math.pow(2, 8));
      result[3] = i - gogo * Math.pow(2, 16) - go * Math.pow(2, 8) + binaryList[3];
      result[2] = go + binaryList[2];
      result[1] = gogo + binaryList[1];
      resultList.push(binaryList[0] + "." + result[1] + "." + result[2] + "." + result[3]);
      continue
    }
    if (i > Math.pow(2, 24) - 1 ) {
      const gogogo = parseInt(i / Math.pow(2, 24));
      const gogo = parseInt(i - gogogo * Math.pow(2, 24) / Math.pow(2, 16));
      const go = parseInt(i - gogogo * Math.pow(2, 24) - gogo * Math.pow(2, 16) / Math.pow(2, 8));
      result[3] = i - gogogo * Math.pow(2,24) -  gogo * Math.pow(2, 16) - go * Math.pow(2, 8) + binaryList[3];
      result[2] = go + binaryList[2];
      result[1] = gogo + binaryList[1];
      result[0] = gogogo + binaryList[0];
      resultList.push(result[0] + "." + result[1] + "." + result[2] + "." + result[3]);
      continue
    }
    const first = binaryList[3] + i;
      resultList.push(binaryList[0] + "." + binaryList[1] + "." + binaryList[2] + "." + first);

  }

      return resultList;  
};  
    



/**查询该用户下的所有设备 */
export const getAllConnect = (ip: string, mask: string) => {
  const subnetMask = calculateSubnets(ip, mask);
  let deviceList = [];
  subnetMask.forEach((item,index) => {
    deviceList.push({
      connectId: index + 1,
      address: item,
      description: "",
      isReachable: "0",
      lastTested: Date.now(),
      ping: undefined,
      connectName: "",
    })
  })
  return {
    success: true,
    data: {
    list: deviceList,
    total: deviceList.length,
    pageSize: 15,
    currentPage: 1,
  }
  }
}






/** 对用户下的设备进行删除 */
export const deleteConnect = (data: { connectIds: Array<number> }) => { 
  const connectList = storageSession().getItem<ConnectSearchResult>('connectList').data.list;
  const customeList = storageSession().getItem<ConnectSearchResult>('connectList').data.customeList;
  const result = customeList.filter((item) => !data.connectIds.includes(item.connectId));
  storageSession().setItem('connectList', {
    success: true,
    data: {
      list: connectList,
      customeList: result,
      total: connectList.length + result.length,
      pageSize: 10,
      currentPage: 1
    }
  })
  return { success: true, data: null };
}


/** 对用户下的设备进行添加 */
export const addConnect = (data: { Connect: Connect }) => { 
  const connectList = storageSession().getItem<ConnectSearchResult>('connectList').data.list;
  const customeList = storageSession().getItem<ConnectSearchResult>('connectList').data.customeList
  if (data.Connect.connectId == 0) {
      let max = 0; 
      customeList.forEach((item) => { if (item.connectId > max) max = item.connectId })
      data.Connect.connectId = max + 1;
  }
  customeList.push(data.Connect);
    storageSession().setItem('connectList', {
    success: true,
    data: {
      list: connectList,
      customeList:customeList,
      total: connectList.length +customeList.length,
      pageSize: 10,
      currentPage: 1
    }
  })
  return { success: true, data: null };
}
/** 对用户下的设备进行修改 */
export const updateConnect = (data: { connectId: number, Connect: Connect }) => { 
  const connectList = storageSession().getItem<ConnectSearchResult>('connectList').data.list;
  const customeList = storageSession().getItem<ConnectSearchResult>('connectList').data.customeList;
  const index = customeList.findIndex((item) => item.connectId === data.connectId);
  customeList[index] = data.Connect;
      storageSession().setItem('connectList', {
    success: true,
    data: {
      list: connectList,
      customeList:customeList,
      total: connectList.length +customeList.length,
      pageSize: 10,
      currentPage: 1
    }
  })
  return { success: true, data: null };
}

type IdentifierResult = {
  success: boolean;
  data: {
    deviceId: string;
  }
}

/** 与后台通信获取后台的唯一标识符 */
export const getIdentifier = (data:string) => {
  return axios.post<IdentifierResult>('http://'+ data + ':8848' +"/get-device-id")
}
