import { testConnect } from "@/api/user/testConnect";
import {
  store,
  defineStore,
  router,
} from "../utils";

import { storageSession } from "@pureadmin/utils";
import { message } from "@/utils/message";

type ConnectType = {
  deviceId: string,
  address: string,
  isReachable: string | null,
  ping: string | null,
}
type ConnectTypeResult = {
  deviceList: Array<ConnectType>,
  loading: boolean,
  timeOutList: Array<ConnectType>,
  connectList: Array<ConnectType>,
}


export const useConnectStore = defineStore({
  id: "pure-connect",
  state: (): ConnectTypeResult => ({
    // 批量操作
    deviceList: storageSession().getItem<ConnectTypeResult>('connectKey')?.deviceList ?? null,
    // 防抖使用
    loading: false,
    // 超时组
    timeOutList: [],
    // 连接组
    connectList: [],
  }),
  actions: {
    /**存储设备的唯一标识符 */
    SET_DEVICELIST(deviceList: Array<ConnectType>) {
      this.deviceList = deviceList;
    },

    /**
     * 此方法用于测试当前地址的连通性
     */
    async handleTestConnect() {
      console.log('sdadsad')
      if (this.loading) return;
      this.loading = true;
      const timeOuts = [];
      const connects = [];
      let count = 0;
      console.log(this.deviceList.length);

      for (let i = 0; i < this.deviceList.length; i++){
        testConnect(this.deviceList.at(i).address).then((res) => {
          if (res.data?.code == 20041) {
            count++;
            this.deviceList[i].isReachable = "1",
              this.deviceList[i].ping = res.data?.data?.ping ?? ""
            connects.push(this.deviceList[i])
          } else {
            count++;
            this.deviceList[i].isReachable = "0",
            this.deviceList[i].ping = "460"  
        }
      }).catch(err => {
        console.log(err);
        count++;
        message(this.deviceList[i].address + '网络连接断开，请重新连接！', { type: 'error' });
        timeOuts.push(this.deviceList[i]);
      }).finally(() => {
        if (count == this.deviceList.length) {
          this.timeOutList = timeOuts;
          console.log(this.timeOutList)
          this.connectList = connects;
          console.log(this.connectList)
        }
      })
      }

      this.loading = false;
    },

    /** 此方法用于初始化 */
    async init() {
      this.SET_DEVICELIST(storageSession().getItem<ConnectTypeResult>('connectKey')?.deviceList ?? "")
      console.log(this.deviceList)
      await this.handleTestConnect();
    },

    skip() {
      this.deviceList.forEach((device, index) => {
        this.timeOutList.forEach((timeOut, i) => {
          if (device.address == timeOut.address) {
            this.deviceList.splice(index, 1);
            this.timeOutList.splice(i, 1);
          }
        })
      })
      if (this.deviceList.length == 0) {

        message("当前未连接任何设备!", { type: "error" });
        router.push({ path: "/testConnect" })
        
      }
    }

    

  }
});

export function useConnectStoreHook() {
  return useConnectStore(store);
}
