
import {
  store,
  defineStore,
} from "../utils";

import { storageSession } from "@pureadmin/utils";
import { useConnectStoreHook } from "./connect";
import { message } from "@/utils/message";
import { getDownloadProgress, getInstallProgress, searchInstallContent, searchInstallStatus } from "@/api/user/installUpgradePackage";


type InstallType = {
  installAddress: string,
  installTitle: string,
  installStatus?: string,
  installProgress?: number,
  startTime?: Date,
}
type InstallTypeResult = {
  installList: Array<InstallType>,
  intervalId: ReturnType<typeof setInterval>;
  loading: boolean,
  timeOutMap: Map<string,number>,
}


export const useInstallStore = defineStore({
  id: "pure-install",
  state: (): InstallTypeResult => ({
    installList: storageSession().getItem<InstallTypeResult>('installKey')?.installList ?? [],
    intervalId:  null,
    loading: false,
    timeOutMap: storageSession().getItem<InstallTypeResult>('installKey')?.timeOutMap ?? [],
  }),
  actions: {


    /**获取所有正在进行的任务的状态 */
    async getAllInstallStatus() {
        // 遍历所有任务，查询状态
      this.installList.forEach((install,index) => {
        getDownloadProgress(install.installAddress).then((res => {
          if (res.data.code == 20041) {

            if (res.data.data.progressStatus == 2 || res.data.data.progressStatus == 3 || res.data.data.progressStatus == 0){
                this.installList.splice(index, 1);
                storageSession().setItem('installKey', {
                installList: this.installList,
              })
              } else {
              install.installProgress = res.data.data.progress;

              storageSession().setItem('installKey', {
                installList: this.installList,
              })

            }
              if (this.installList.length == 0) {
                message("下载任务已完成!", { type: "success" });
                this.timeOutMap = null;
                storageSession().setItem('installKey', {
                  installList: this.installList,
                  timeOutMap: this.timeOutMap
                })
              }

          } else {
            message(install.installAddress + "查询下载状态失败!", { type: "error" }); 
            this.installList.splice(index, 1);
              storageSession().setItem('installKey', {
                installList: this.installList,
              })
            }
        })).catch(err => {
          message(install.installAddress + "连接超时!");
          console.log(err);
          useConnectStoreHook().handleTestConnect();
          if (this.timeOutMap.get(install.installAddress) >= 3) {
            this.timeOutMap.remove(install.installAddress);
            this.installList.splice(index, 1);
          } else {
            this.timeOutMap.set(install.installAddress,this.timeOutMap.get(install.installAddress)+1)
          }
          storageSession().setItem('installKey', {
              installList: this.installList,
              timeOutMap: this.timeOutMap
            })
        })
        })

    },
    
    /** 此方法用于初始化 */
    async init() {
      if (this.installList.length == 0) {
        clearInterval(this.intervalId);
        this.intervalId = null;
      }
      if (this.loading) {
        return;
      }
      this.loading = true;

      await this.getAllInstallStatus();
      
      this.loading = false;
      
    },
    
    async debounceInit() {
      console.log(this.intervalId)
      if (this.intervalId == null) {
        this.installList = storageSession().getItem<InstallTypeResult>('installKey')?.installList ?? [];
        this.timeOutMap = storageSession().getItem<InstallTypeResult>('installKey')?.timeOutMap ?? [];
        if (this.installList.length != 0) {
          this.intervalId = setInterval(this.init, 3000);  
        }         
      }
    }

  }
  


}
);

export function useInstallStoreHook() {
  return useInstallStore(store);
}
