
import { getTask, getTaskProgress, getUnfinishedTask, transformStatusToFail, transfromTaskStatus } from "@/api/user/task";
import {
  store,
  defineStore,
} from "../utils";

import { storageSession } from "@pureadmin/utils";
import { useConnectStoreHook } from "./connect";
import { message } from "@/utils/message";
import { useUserStoreHook } from "./user";


type TaskType = {
  taskAddress: string,
  taskTitle: string,
  taskContent: string,
  taskStatus?: string,
  taskProgress?: number,
  startTime?: Date,
}
type TaskTypeResult = {
  taskList: Array<TaskType>,
  intervalId: ReturnType<typeof setInterval>;
  loading: boolean,
}


export const useTaskStore = defineStore({
  id: "pure-task",
  state: (): TaskTypeResult => ({
    taskList: storageSession().getItem<TaskTypeResult>('taskKey')?.taskList ?? [],
    intervalId: null,
    loading: false,
  }),
  actions: {
    /**获取所有正在进行的任务的状态 */
    async getAllTaskStatus() {
        // 遍历所有任务，查询状态
        getTaskProgress({ title: this.taskList[0].taskTitle,userId: useUserStoreHook().ID,role: useUserStoreHook().roles}).then(res => {
          if (res.success) {
            res.data.list.forEach(item => {

              this.taskList.forEach((task, index) => {

                if (item.address == task.taskAddress) {
                  console.log('dsadasd')
                  if (item.status == '安装成功' || item.status == '安装失败' || item.status == '安装异常') {
                    this.taskList.splice(index, 1);
                    storageSession().setItem('taskKey', {
                      taskList: this.taskList,
                    })
                  }
                }
                if (this.taskList.length == 0) {
                  message("安装任务已完成!", { type: "success" });
                }
              })
            })
          } else {
            message("无法同步安装状态!", { type: "error" });
              clearInterval(this.intervalId);
              this.intervalId = null;
              storageSession().setItem('taskKey', {
              taskList: [],
              })
          }
        }).catch((err) =>{
          console.log(err);
          message("与远程服务器连接超时！", { type: "error" });
          clearInterval(this.intervalId);
              this.intervalId = null;
              storageSession().setItem('taskKey', {
              taskList: [],
              })
          
        })

    },
    
    /** 此方法用于初始化 */
    async init() {
      if (this.taskList.length == 0) {
        clearInterval(this.intervalId);
        this.intervalId = null;
      }
      if (this.loading) {
        return;
      }
      this.loading = true;

      await this.getAllTaskStatus();
      
      this.loading = false;
      
    },
    
    async debounceInit() {
      if(this.intervalId == null) {
        this.taskList = storageSession().getItem<TaskTypeResult>('taskKey')?.taskList ?? [];

        console.log(this.taskList);
        if (this.taskList.length != 0) {

          this.intervalId = setInterval(this.init, 1000);

        }

      }
    }

  }
  


}
);

export function useTaskStoreHook() {
  return useTaskStore(store);
}
