import Cookies from "js-cookie";
import { storageLocal, storageSession } from "@pureadmin/utils";
import { useUserStoreHook } from "@/store/modules/user";
import { getMine } from "@/api/user";
import { userInfo } from "os";
import data from "@iconify-icons/ri/fullscreen-fill";
// 用户有两个状态
// 1.白板状态、没有token、localstorage没有信息
// 2.登录状态、有token、localstorage有信息

// 未登录状态的所有可能操作
// 登录（白板->登录）getLogin
// 登录状态所有可能状态变化的操作
// token过期无感刷新（登录->（localStorage信息会刷新）登录）refreshTokenApi
// token过期(登录->（localStorage信息仍然存在、会被刷新）特殊白板)getLogin
// 用户登出（登录->白板）getLogin
// 关闭网页（登录->（token仍然存在、localStorage信息不存在、需重新登录、相当于白板）白板）getLogin
export interface DataInfo<T> {
  /** 用户Id */
  ID?: number,
  /** 头像 */
  avatar?: string;
  /** 用户名 */
  username?: string;
  /** 昵称 */
  nickname?: string;
  /** 当前登录用户的角色 */
  roles?: Array<string>;
  /** 当前登录用户的邮箱 */
  email?: string;
  /** 当前登录用户的电话号 */
  phone?: string;
  /** 当前登录用户的个人界面背景图片 */
  backgroundImg: string;
  /** 当前登录用户的个人介绍 */
  introduction: string;
}

export interface TokenInfo<T> {
  /** token */
  accessToken: string;
  /** `accessToken`的过期时间（时间戳） */
  expires: T;
  /** 用于调用刷新accessToken的接口时所需的token */
  refreshToken: string;
}
export const userKey = "user-info";
export const TokenKey = "authorized-token";
/**
 * 通过`multiple-tabs`是否在`cookie`中，判断用户是否已经登录系统，
 * 从而支持多标签页打开已经登录的系统后无需再登录。
 * 浏览器完全关闭后`multiple-tabs`将自动从`cookie`中销毁，
 * 再次打开浏览器需要重新登录系统
 * */
export const multipleTabsKey = "multiple-tabs";

/** 获取`token` */
export function getToken(): TokenInfo<number> {
  // 此处与`TokenKey`相同，此写法解决初始化时`Cookies`中不存在`TokenKey`报错
  return Cookies.get(TokenKey)
    ? JSON.parse(Cookies.get(TokenKey))
    : storageLocal().getItem(userKey);
}

/**
 * @description 设置`token`以及一些必要信息并采用无感刷新`token`方案
 * 无感刷新：后端返回`accessToken`（访问接口使用的`token`）、`refreshToken`（用于调用刷新`accessToken`的接口时所需的`token`，`refreshToken`的过期时间（比如30天）应大于`accessToken`的过期时间（比如2小时））、`expires`（`accessToken`的过期时间）
 * 将`accessToken`、`expires`、`refreshToken`这三条信息放在key值为authorized-token的cookie里（过期自动销毁）
 * 将`avatar`、`username`、`nickname`、`roles`、`refreshToken`、`expires`这六条信息放在key值为`user-info`的localStorage里（利用`multipleTabsKey`当浏览器完全关闭后自动销毁）
 */
export async function setToken(data: TokenInfo<Date>) {
  console.log("setToken被调用");
  let expires = 0;
  let { accessToken, refreshToken } = data;
  const { isRemembered, loginDay } = useUserStoreHook();
  console.log(refreshToken)
  if (refreshToken === undefined) {
    console.log("refreshToken为空");
    refreshToken = JSON.parse(Cookies.get(TokenKey)).refreshToken;
  }
  expires = new Date(data.expires).getTime(); // 如果后端直接设置时间戳，将此处代码改为expires = data.expires，然后把上面的DataInfo<Date>改成DataInfo<number>即可
  const cookieString = JSON.stringify({ accessToken, expires, refreshToken });
  console.log(expires)
  // expires > 0
  //   ? Cookies.set(TokenKey, cookieString, {
  //     expires: (expires - Date.now()) / 86400000
  //   })
  //   : 
Cookies.set(TokenKey, cookieString);
  Cookies.set(
    multipleTabsKey,
    "true",
    isRemembered
      ? {
        expires: loginDay
      }
      : {}
  );
  storageLocal().setItem(userKey, {
    refreshToken,
    expires,
  });
  refreshUserInfo();
}
 /**
   * 此方法用于更新pinia
   * @param param0 
   */
  function setUserKey({ avatar, username, nickname, roles,ID,email,phone,backgroundImg,introduction}) {
    useUserStoreHook().SET_ID(ID);
    useUserStoreHook().SET_AVATAR(avatar);
    useUserStoreHook().SET_USERNAME(username);
    useUserStoreHook().SET_NICKNAME(nickname);
    useUserStoreHook().SET_ROLES(roles);
    useUserStoreHook().SET_EMAIL(email);
    useUserStoreHook().SET_PHONE(phone);
    useUserStoreHook().SET_BACKGROUNDIMG(backgroundImg);
    useUserStoreHook().SET_INTRODUCTION(introduction);

    storageLocal().setItem(userKey, {
      avatar,
      username,
      nickname,
      roles,
      ID,
      email,
      phone,
      backgroundImg,
      introduction
    });
}
  /**
   * 此方法用于刷新用户信息
   */
  export async function refreshUserInfo() {
    //在得到token值后、获取用户信息、用于主页展示
  const { data: UserInfo } = await getMine();
    // 成功获取用户信息、进行pinia存储
  if (UserInfo.roles && UserInfo.ID) {
    // 
    const { username, roles, ID } = UserInfo;
    console.log({ username, roles, ID })
    setUserKey({
      avatar: UserInfo?.avatar ?? "",
      username,//必须
      nickname: UserInfo?.nickname ?? "",
      roles,//必须
      ID,//必须
      email: UserInfo?.email ?? "",
      phone: UserInfo?.phone ?? "",
      backgroundImg: UserInfo?.backgroundImg ?? "",
      introduction: UserInfo?.introduction ??"",
    });
  } else {
    //未成功获取、从原有本地Local中获取
    const avatar =
      storageLocal().getItem<DataInfo<number>>(userKey)?.avatar ?? "";
    const username =
      storageLocal().getItem<DataInfo<number>>(userKey)?.username ?? "";
    const nickname =
      storageLocal().getItem<DataInfo<number>>(userKey)?.nickname ?? "";
    const roles =
      storageLocal().getItem<DataInfo<number>>(userKey)?.roles ?? [];
    const ID =
      storageLocal().getItem<DataInfo<number>>(userKey)?.ID ?? 0;
    const email = 
      storageLocal().getItem<DataInfo<number>>(userKey)?.email ?? 0;
    const phone =
      storageLocal().getItem<DataInfo<number>>(userKey)?.phone ?? 0;
    const backgroundImg =
      storageLocal().getItem<DataInfo<number>>(userKey)?.backgroundImg ?? 0;
    const introduction = 
      storageLocal().getItem<DataInfo<number>>(userKey)?.introduction ?? 0;
    setUserKey({
      avatar,
      username,
      nickname,
      roles,
      ID,
      email,
      phone,
      backgroundImg,
      introduction,
    });
  }
  }
/** 删除`token`以及key值为`user-info`的localStorage信息 */
export function removeToken() {
  console.log("removeToken执行")
  Cookies.remove(TokenKey);
  Cookies.remove(multipleTabsKey);
  storageLocal().removeItem(userKey);
  storageSession().removeItem('connectKey');
  storageSession().removeItem('installKey');
  storageSession().removeItem('taskKey');
}

/** 格式化token（jwt格式） */
export const formatToken = (token: string): string => {
  return "Bearer " + token;
};
