import "./reset.css";
import dayjs from "dayjs";
import roleForm from "../form/role.vue";
import editForm from "../form/index.vue";
import { zxcvbn } from "@zxcvbn-ts/core";
import { handleTree } from "@/utils/tree";
import { message } from "@/utils/message";
import userAvatar from "@/assets/user.jpg";
import { usePublicHooks } from "../../hooks";
import { addDialog } from "@/components/ReDialog";
import type { PaginationProps } from "@pureadmin/table";
import ReCropperPreview from "@/components/ReCropperPreview";
import type { FormItemProps, RoleFormItemProps } from "../utils/types";


import {
  getKeyList,
  isAllEmpty,
  hideTextAtIndex,
  deviceDetection,
  storageSession
} from "@pureadmin/utils";
import {
  type Ref,
  h,
  ref,
  toRaw,
  watch,
  computed,
  reactive,
  onMounted,
  nextTick
} from "vue";
import { addConnect, deleteConnect, getAllConnect, testConnect, updateConnect } from "@/api/user/testConnect";
import { useUserStoreHook } from "@/store/modules/user";
import { connect } from "echarts";
import { FormInstance } from "element-plus";


export function useUser(tableRef: Ref, treeRef: Ref) {
  const form = reactive({
    address: "",
    subnetMask: "",
  });
  const formRef = ref<FormInstance>();
  const dataList = ref([]);
  const loading = ref(false);
  // 上传头像信息
  const treeData = ref([]);
  const treeLoading = ref(true);
  const selectedNum = ref(0);
  const pagination = reactive<PaginationProps>({
    total: 0,
    pageSize: 10,
    currentPage: 1,
    background: true
  });
  const selectRow = ref([]);
  const columns: TableColumnList = [
    {
      label: "勾选列", // 如果需要表格多选，此处label必须设置
      type: "selection",
      fixed: "left",
      reserveSelection: true // 数据刷新后保留选项
    },
    {
      label: "设备编号",
      prop: "connectId",
      width: 90
    },
    {
      label: "ip地址",
      prop: "address",
      minWidth: 130
    },
    {
      label: "连通性",
      prop: "isReachable",
      minWidth: 90,
      cellRenderer: ({ row, props }) => (
        <div>
          <el-text v-loading={row.ping == undefined} v-show={row.ping == undefined} style="color: transparent;">hhh</el-text>
          <el-tag
            v-show={row.ping != undefined} 
          size={props.size}
          type={row.isReachable === '1' ? "success" : "danger"}
          effect="plain"
        >
            {row.isReachable === '1' ? "已连通" : "未连通"}
        </el-tag>
        
        </div>

      )
    },
    {
      label: "延迟",
      prop: "ping",
      minWidth: 90,
      cellRenderer: ({ row, props }) => (
        <div>
          <el-text
          v-show={row.ping != undefined}
          size={props.size}
          effect="plain"
        >
          {row.ping + 'ms'}
          </el-text>
          <el-text v-loading={row.ping == undefined} v-show={row.ping == undefined} style="color: transparent;background-color: transparent !important" >hhh</el-text>
        </div>
        
      )
    },
    {
      label: "最后测试时间",
      minWidth: 130,
      prop: "lastTested",
      formatter: ({ lastTested }) =>
        dayjs(lastTested).format("YYYY-MM-DD HH:mm:ss")
    },
    // {
    //   label: "操作",
    //   fixed: "right",
    //   width: 220,
    //   slot: "operation"
    // }
  ];
  const buttonClass = computed(() => {
    return [
      "!h-[20px]",
      "reset-margin",
      "!text-gray-500",
      "dark:!text-white",
      "dark:hover:!text-primary"
    ];
  });

  function handleUpdate(row) {
    console.log(row);
  }

  /**删除设备 */
  function handleDelete(row) {
    const { success } = deleteConnect({ connectIds: [row.connectId] })

      if (success) {
         message(`您删除了设备编号为${row.connectId}的这条数据`, { type: "success" });
      } else {
        message('删除失败', { type: "error" });
      }

  }

  function handleSizeChange(val: number) {
    console.log(`${val} items per page`);
  }

  function handleCurrentChange(val: number) {
    console.log(`current page: ${val}`);
  }

  /** 当CheckBox选择项发生变化时会触发该事件 */
  function handleSelectionChange(val) {
    console.log(val);
    selectRow.value = val;
    console.log(selectedNum.value)
    const curSelected = tableRef.value.getTableRef().getSelectionRows();
    console.log(curSelected)

    selectedNum.value = curSelected.length
    // 重置表格高度
    tableRef.value.setAdaptive();

  }

  function handleSelect(selection, row) {
    console.log(selection, row);

    if (row.ping == undefined) {
      tableRef.value.getTableRef().toggleRowSelection(row, false);
      return
    }

    if (row.isReachable == "0") {
      tableRef.value.getTableRef().toggleRowSelection(row, false);

      let i = 0

      dataList.value.forEach((item, index) => {
        if (item.connectId == row.connectId) {
          i = index;
          dataList.value[index].ping = undefined;
        }
      })

      testConnect(row.address).then(
        res => {
          if (res.data?.code == 20041) {
            dataList.value.at(i).isReachable = '1';
            dataList.value.at(i).ping = res.data?.data?.ping;
            tableRef.value.getTableRef().toggleRowSelection(row, true);
          } else {
            message("无法连接，勾选失败!", { type: 'error' });
            dataList.value.at(i).isReachable = '0';
            dataList.value.at(i).ping = '460';

          }
        }
      ).catch(err => {
        message("无法连接，勾选失败!", { type: 'error' });
        console.log(err)
        dataList.value.at(i).isReachable = '0';
        dataList.value.at(i).ping = '460';

      })

    }
 }

  /** 取消选择 */
  function onSelectionCancel() {
    selectedNum.value = 0;
    // 用于多选表格，清空用户的选择
    tableRef.value.getTableRef().clearSelection();
  }

  /** 批量删除 */
  function onbatchDel() {
    // 返回当前选中的行
    const curSelected = tableRef.value.getTableRef().getSelectionRows();
    // 接下来根据实际业务，通过选中行的某项数据，比如下面的id，调用接口进行批量删除
    const { success } =deleteConnect({ connectIds: getKeyList(curSelected, "id") })
    if (success) {
      message(`已删除用户编号为 ${getKeyList(curSelected, "id")} 的数据`, {
        type: "success"
      });
    } else {
      message("删除失败!", { type: "error" });
    }
    tableRef.value.getTableRef().clearSelection();

  }

 

  /** 获取当前用户下的所有设备 */
  async function onSearch(formEL) {
    if (loading.value) {
      return;
    }
    loading.value = true;
    if (!formEL) {
      console.log(formEL);
      loading.value = false;
      return;
    }
    console.log(formEL)
    await formEL.validate(async (valid, fields) => { 
      if (valid) {
        const { data } = getAllConnect("192.168.33.121", form.subnetMask);
        dataList.value = data.list;
        console.log(dataList.value)
        /**测试所有设备的连通性 */
  


         dataList.value.forEach((item, index) => {
            testConnect(item.address).then(
              res => {
                if (res.data?.code == 20041) {
                  dataList.value.at(index).isReachable = '1';
                  dataList.value.at(index).ping = res.data?.data?.ping;
                  setTimeout(() => {
                    tableRef.value.getTableRef().toggleRowSelection(dataList.value.find((i) => { return i.connectId == item.connectId }), true);
                  }, 300)
                } else {
                  dataList.value.at(index).isReachable = '0';
                  dataList.value.at(index).ping = '460';
                }
              }
            ).catch(err => {
              console.log(err)
              dataList.value.at(index).isReachable = '0';
              dataList.value.at(index).ping = '460';
            })
          })



          
          




          pagination.total = data.total;
          pagination.pageSize = data.pageSize;
          pagination.currentPage = data.currentPage;

      } else {
        message("地址或子网掩码不符合规范！", { type: 'error' });
      }
    })
    loading.value = false;
  }

  const resetForm = formEl => {
    if (!formEl) return;
    formEl.resetFields();
    treeRef.value.onTreeReset();
    onSearch(formEl);
  };



  function openDialog(title = "新增", row?: FormItemProps) {
    addDialog({
      title: `${title}设备`,
      props: {
        formInline: {
          title,
          connectId: row?.connectId ?? 0,
          connectName: row?.connectName ?? "",
          description: row?.description ?? "",
          address: row?.address ?? "",
        }
      },
      width: "46%",
      draggable: true,
      fullscreen: deviceDetection(),
      fullscreenIcon: true,
      closeOnClickModal: false,
      contentRenderer: () => h(editForm, { ref: formRef }),
      beforeSure: (done, { options }) => {
        const FormRef = formRef.value.getRef();
        const curData = options.props.formInline as FormItemProps;
        function chores() {
          message(`您${title}了设备名称为${curData.connectName}的这条数据`, {
            type: "success"
          });
          done(); // 关闭弹框
        }
        FormRef.validate(valid => {
          if (valid) {
            console.log("curData", curData);
            // 表单规则校验通过
            if (title === "新增") {
              const {success} =  addConnect({ Connect: curData })
              if (success) {
                chores();
              } else {
                message("添加设备失败!", { type: "error" });
              }

            } else {

             const {success} =  updateConnect({ connectId: curData.connectId, Connect: curData })
                if (success) {
                  chores();
                } else {
                  message("更新设备信息失败!", { type: "error" });
                }
            }
          }
        });
      }
    });
  }




  onMounted(async () => {
    treeLoading.value = true;
  });

  return {
    form,
    loading,
    columns,
    dataList,
    treeData,
    treeLoading,
    selectedNum,
    pagination,
    buttonClass,
    deviceDetection,
    onSearch,
    resetForm,
    onbatchDel,
    openDialog,
    handleUpdate,
    handleDelete,
    handleSizeChange,
    onSelectionCancel,
    handleCurrentChange,
    handleSelectionChange,
    handleSelect,
    selectRow
  };
}
