import dayjs from "dayjs";
import editForm from "../form.vue";
import { handleTree } from "@/utils/tree";
import { message } from "@/utils/message";
import { ElMessageBox } from "element-plus";
import { usePublicHooks } from "../hooks";
import { transformI18n } from "@/plugins/i18n";
import { addDialog } from "@/components/ReDialog";
import type { FormItemProps } from "../utils/types";
import type { PaginationProps } from "@pureadmin/table";
import { getKeyList, deviceDetection, storageSession } from "@pureadmin/utils";
import { getRoleList, getRoleMenu, getRoleMenuIds } from "@/api/system";
import { type Ref, reactive, ref, onMounted, h, toRaw, watch } from "vue";
import { useConnectStore, useConnectStoreHook } from "@/store/modules/connect";
import { searchInstallContent } from "@/api/user/installUpgradePackage";
import { useUserStoreHook } from "@/store/modules/user";
import { useTaskStoreHook } from "@/store/modules/task";
import { addNewTask, addNewTaskByInitrd, addNewTaskByKernel } from "@/api/user/task";
import { func } from "vue-types";

export function useRole(treeRef: Ref) {


  const form = reactive({
    packageName: "",
    installType: "",
  });



  const curRow = ref();
  const formRef = ref();
  const dataList = ref([]);
  const treeIds = ref([]);
  const treeData = ref([]);
  const isShow = ref(false);
  const loading = ref(true);
  const isLinkage = ref(false);
  const treeSearchValue = ref();
  const isExpandAll = ref(false);
  const isSelectAll = ref(false);
  const treeProps = {
    value: "id",
    label: "title",
    children: "children"
  };
  const pagination = reactive<PaginationProps>({
    total: 0,
    pageSize: 10,
    currentPage: 1,
    background: true
  });
  const columns: TableColumnList = [
    {
      type: "expand",
      slot: "expand"
    },
    {
      label: "升级包名",
      prop: "packageName"
    },
    {
      label: "操作",
      fixed: "right",
      width: 220,
      slot: "operation"
    }
  ];

  const childColumns: TableColumnList = [
    {
      label: "设备ip",
      prop: "address"
    },
    {
      label: "状态",
      cellRenderer: ({  props }) => (
        <el-tag
          size={props.size}
          type={'primary'}
          effect="plain"
        >
          {'已下载'}
        </el-tag>
      ),
    }
  ];


  function handleDelete(row) {
    message(`您删除了角色名称为${row.name}的这条数据`, { type: "success" });
    onSearch();
  }

  function handleSizeChange(val: number) {
    console.log(`${val} items per page`);
  }

  function handleCurrentChange(val: number) {
    console.log(`current page: ${val}`);
  }

  function handleSelectionChange(val) {
    console.log("handleSelectionChange", val);
  }


  const selectRow = ref();
  const timeOutList = ref();
  const dialogTableVisible = ref(false);
  const noInstallList = ref([])

  const dialogFormVisible = ref(false);


  function install(row) {

    if (useTaskStoreHook().taskList.length != 0) {
      message("当前存在安装进程！请稍后重试", { type: "error" });
      return;
    }

    selectRow.value = row
    console.log(selectRow.value)
    // 先检查选中的安装的升级包当前连接的设备是否全部包含
    const deviceList = useConnectStoreHook().deviceList;
    console.log(deviceList);
    const installList = row.family;

    noInstallList.value = [];


    let array = [];

    deviceList.forEach((item) => {
      installList.forEach((install, index) => {
        console.log(install)
        if (install.address == item.address) {
          array.push(item);
        }
      })
    })

    noInstallList.value = deviceList.filter((device) => { !array.includes(device) })

    console.log(deviceList);
    console.log(noInstallList.value)

    if (noInstallList.value.length == 0) {
      // 当前升级包全部ip已经下载

      // 测试连通性

      useConnectStoreHook().handleTestConnect();
      timeOutList.value = useConnectStoreHook().timeOutList;

      if (timeOutList.value.length > 0) {
        dialogTableVisible.value = true;
        return;
      }





      ElMessageBox.confirm(
        '是否安装名为' + row.packageName + "的升级包",
        '安装',
        {
          confirmButtonText: '安装',
          cancelButtonText: '取消',
          type: 'warning',
        }
      ).then(() => {
        
        // 安装

        dialogFormVisible.value = true;

      }).catch(() =>{
        
  

      })

    } else {
      // 当前升级包部分ip未下载

      dialogVisible.value = true;


    }

  }

  function handleConfirm(row) {
    dialogFormVisible.value = false;

    // 向所有设备发送安装请求

    const deviceList = useConnectStoreHook().deviceList;
    const taskList = []
    console.log(form.installType);
    if (form.installType == 'kernel') {
      const title = "" + useUserStoreHook().ID + Date.now();

      deviceList.forEach((device) => {
        taskList.push({
            taskAddress: device.address,
            taskTitle: title,//用户名+ 时间戳 - 唯一标识此日志集合
            taskContent: selectRow.value.packageName,
            taskStatus: "安装中",
            taskProgress: 0,
            startTime: Date.now()
          })
      })

      storageSession().setItem('taskKey', {
        taskList: taskList,
      })
      useTaskStoreHook().debounceInit();
      if (taskList.length > 0) {
        message("开始安装",{type:"success"});
      }
      

      deviceList.forEach(device => {
      console.log(device)
      addNewTaskByKernel(device.address, selectRow.value.packageName,title, useUserStoreHook().ID, useUserStoreHook().roles,).then(res => {
        if (res.data.code == 20011) {
          message(device.address + "设备安装成功!");
        } else {
          message(device.address + "设备被占用！请稍后重试");
          const taskList = useTaskStoreHook().taskList;
          taskList.forEach((task,index) => {
            if (task.taskAddress == device.address) {
              taskList.splice(1, index);
            }
          })
        }
      }).catch((err) => {
        console.error(err);
        message(device.address + "设备超时！");
        const taskList = useTaskStoreHook().taskList;
        taskList.forEach((task, index) => {
          if (task.taskAddress == device.address) {
            taskList.splice(1, index);
          }
        })
      })
    })
    }


    if (form.installType == 'initrd') {
      deviceList.forEach(device => {
        const title = "" + useUserStoreHook().ID + Date.now();
        deviceList.forEach((device) => {
          taskList.push({
            taskAddress: device.address,
            taskTitle: title,//用户名+ 时间戳 - 唯一标识此日志集合
            taskContent: selectRow.value.packageName,
            taskStatus: "安装中",
            taskProgress: 0,
            startTime: Date.now()
          })
        })

        storageSession().setItem('taskKey', {
          taskList: taskList,
        })
        useTaskStoreHook().debounceInit();

        addNewTaskByInitrd(device.address, row.packageName, title, useUserStoreHook().ID, useUserStoreHook().roles,).then(res => {
          if (res.data.code == 20011) {
            message(device.address + "设备安装成功!");
          } else {
            message(device.address + "设备被占用！请稍后重试");
            const taskList = useTaskStoreHook().taskList;
            taskList.forEach((task, index) => {
              if (task.taskAddress == device.address) {
                taskList.splice(1, index);
              }
            })
          }
        }).catch(() => {
          message(device.address + "设备超时！");
          const taskList = useTaskStoreHook().taskList;
          taskList.forEach((task, index) => {
            if (task.taskAddress == device.address) {
              taskList.splice(1, index);
            }
          })
        })
      })
    }

  }

  function handleCancel() {
    dialogFormVisible.value = false;
  }

  async function onSearch() {
    loading.value = true;

    const deviceList = useConnectStoreHook().deviceList;

    const map = new Map<string,Array<object>>();

    let count = 0;

    deviceList.forEach((device) => {
      searchInstallContent(device.address).then((res) => {
        if (res.data.code == 20041) {
          const installList = res.data.data.list;
          installList.forEach((install) => {
            if (map.get(install) != undefined && map.get(install) != null) {
              map.get(install).push({ address: device.address });
            } else {
              const family = [{address : device.address }];
              map.set(install, family);
              console.log(map.size)
            }
          })
        } else {
          message("获取" + device.address + "已下载资源失败!",{type:"error"});
        }
      }).catch(err => {
        console.log(err);
        useConnectStoreHook().handleTestConnect();
        message("连接" + device.address + "超时！请检查");
      }).finally(() => {
        count++;
        if (count == deviceList.length) {
          console.log(map.size);
          const datas = []
          map.forEach((value, key) => {
            console.log(value, key);
            datas.push({
              packageName: key,
              family: value
            })
          })
          console.log(datas)
          dataList.value = datas;
          pagination.total = datas.length;
          pagination.pageSize = 10;
          pagination.currentPage = 1;

        }
      }) 
    })

    setTimeout(() => {
      loading.value = false;
    }, 500);
  }

  const resetForm = formEl => {
    if (!formEl) return;
    formEl.resetFields();
    onSearch();
  };

  const dialogVisible = ref(false);















  function openDialog(title = "新增", row?: FormItemProps) {
    addDialog({
      title: `${title}角色`,
      props: {
        formInline: {
          name: row?.name ?? "",
          code: row?.code ?? "",
          remark: row?.remark ?? ""
        }
      },
      width: "40%",
      draggable: true,
      fullscreen: deviceDetection(),
      fullscreenIcon: true,
      closeOnClickModal: false,
      contentRenderer: () => h(editForm, { ref: formRef }),
      beforeSure: (done, { options }) => {
        const FormRef = formRef.value.getRef();
        const curData = options.props.formInline as FormItemProps;
        function chores() {
          message(`您${title}了角色名称为${curData.name}的这条数据`, {
            type: "success"
          });
          done(); // 关闭弹框
          onSearch(); // 刷新表格数据
        }
        FormRef.validate(valid => {
          if (valid) {
            console.log("curData", curData);
            // 表单规则校验通过
            if (title === "新增") {
              // 实际开发先调用新增接口，再进行下面操作
              chores();
            } else {
              // 实际开发先调用修改接口，再进行下面操作
              chores();
            }
          }
        });
      }
    });
  }

  /** 菜单权限 */
  async function handleMenu(row?: any) {
    const { id } = row;
    if (id) {
      curRow.value = row;
      isShow.value = true;
      const { data } = await getRoleMenuIds({ id });
      treeRef.value.setCheckedKeys(data);
    } else {
      curRow.value = null;
      isShow.value = false;
    }
  }

  /** 高亮当前权限选中行 */
  function rowStyle({ row: { id } }) {
    return {
      cursor: "pointer",
      background: id === curRow.value?.id ? "var(--el-fill-color-light)" : ""
    };
  }

  /** 菜单权限-保存 */
  function handleSave() {
    const { id, name } = curRow.value;
    // 根据用户 id 调用实际项目中菜单权限修改接口
    console.log(id, treeRef.value.getCheckedKeys());
    message(`角色名称为${name}的菜单权限修改成功`, {
      type: "success"
    });
  }

  /** 数据权限 可自行开发 */
  // function handleDatabase() {}

  const onQueryChanged = (query: string) => {
    treeRef.value!.filter(query);
  };

  const filterMethod = (query: string, node) => {
    return transformI18n(node.title)!.includes(query);
  };

  onMounted(async () => {
    onSearch();
    // const { data } = await getRoleMenu();
    // treeIds.value = getKeyList(data, "id");
    // treeData.value = handleTree(data);
  });

  watch(isExpandAll, val => {
    val
      ? treeRef.value.setExpandedKeys(treeIds.value)
      : treeRef.value.setExpandedKeys([]);
  });

  watch(isSelectAll, val => {
    val
      ? treeRef.value.setCheckedKeys(treeIds.value)
      : treeRef.value.setCheckedKeys([]);
  });

  return {
    form,
    isShow,
    curRow,
    loading,
    columns,
    rowStyle,
    dataList,
    treeData,
    treeProps,
    isLinkage,
    pagination,
    isExpandAll,
    isSelectAll,
    treeSearchValue,
    // buttonClass,
    onSearch,
    resetForm,
    openDialog,
    handleMenu,
    handleSave,
    handleDelete,
    filterMethod,
    transformI18n,
    onQueryChanged,
    // handleDatabase,
    handleSizeChange,
    handleCurrentChange,
    handleSelectionChange,
    childColumns,
    install,
    dialogTableVisible,
    timeOutList,
    selectRow,
    dialogVisible,
    noInstallList,
    dialogFormVisible,
    handleCancel,
    handleConfirm
  };
}
