import { delay } from "@pureadmin/utils";
import { ref, onMounted, reactive } from "vue";
import type { PaginationProps } from "@pureadmin/table";
import ThumbUp from "@iconify-icons/ri/thumb-up-line";
import Hearts from "@iconify-icons/ri/hearts-line";
import Empty from "./empty.svg?component";
import { getDepolyUpgradePackageList } from "@/api/user/deployUpgradePackage";
import { message } from "@/utils/message";
import { useConnectStoreHook } from "@/store/modules/connect";

export function useColumns() {
  const dataList = ref([]);
  const loading = ref(true);
  const columns: TableColumnList = [
    {
      sortable: true,
      label: "序号",
      prop: "packageId"
    },
    {
      sortable: true,
      label: "服务器名称",
      prop: "packageName",

    },
    {
      sortable: true,
      label: "版本号",
      prop: "packageVersion"
    },
    {
      sortable: true,
      label: "文件大小",
      prop: "packageSize"
    },
    {
      sortable: true,
      label: "发布时间",
      minWidth: 100,
      prop: "createTime",
    },
    {
      sortable: true,
      label: "状态",
      prop: "status",
      cellRenderer: ({ row, props }) => (
        <el-tag
          size={props.size}
          type={getPackageStatus(row.status)}
          effect="plain"
        >
          {getPackageStatus(row.status, true)}
        </el-tag>
      )
    },
    {
      label: "查看",
      fixed: "right",
      slot: "operation"
    }
  ];

    const getPackageStatus = (type, text = false) => {
    switch (type) {
      case 0:
        return text ? "最新版本" : "success";
      case 1:
        return text ? "有新版本" : "primary";
    }
  };
  /** 分页配置 */
  const pagination = reactive<PaginationProps>({
    pageSize: 10,
    currentPage: 1,
    layout: "prev, pager, next",
    total: 0,
    align: "center"
  });

  function onCurrentChange(page: number) {
    console.log("onCurrentChange", page);
    loading.value = true;
    delay(300).then(() => {
      loading.value = false;
    });
  }
  const datas = ref();

  // 此方法用于获取已安装的升级包
  /**如果后台无法获取是否已安装的更新包需要更新，则获取后再跳一个接口进行查询 */
  const hanldeGetDeployUpgradePackageList = () => { 
    getDepolyUpgradePackageList().then(res => {
      if (res.success) {
        dataList.value = [res.data.list]
        console.log(dataList.value)
      } else {
        message("获取已安装的升级包失败!", { type: "error" });
      }
    }).catch(err => {
      console.log(err);
      useConnectStoreHook().handleTestConnect();
    })
  }

  onMounted(() => {
    hanldeGetDeployUpgradePackageList();
    pagination.total = dataList.value.length;
    loading.value = false;
  });

  function download(row) {
    var a = document.createElement("a");
    var event = new MouseEvent("click");
    a.download = row.packageName;
    a.href = row.ossPath;
    a.dispatchEvent(event);
  }


  return {
    Empty,
    loading,
    columns,
    dataList,
    pagination,
    onCurrentChange,
    download,
    datas
  };
}
