import { dayjs, cloneDeep, getRandomIntBetween } from "./utils";
import GroupLine from "@iconify-icons/ri/group-line";
import Question from "@iconify-icons/ri/question-answer-line";
import CheckLine from "@iconify-icons/ri/chat-check-line";
import Smile from "@iconify-icons/ri/star-smile-line";
import { onMounted, ref } from "vue";
import { getDownloadData, getFeedbackDynamic, getFeedbackSolved, getFeedbackSummary, getInstallData, getPackagePassRate, getPackageSummary, getTableData } from "@/api/statistic";
import { message } from "@/utils/message";

const days = ["周日", "周一", "周二", "周三", "周四", "周五", "周六"];


/** 总上传升级包数量、用户反馈数量、用户反馈解决数量、升级包校验通过率 */
const chartData = ref([
  {
    icon: GroupLine,
    bgColor: "#effaff",
    color: "#41b6ff",
    duration: 2200,
    name: "升级包数",
    value: "",
    percent: "",
    data: []// 平滑折线图数据
  },
  {
    icon: Question,
    bgColor: "#fff5f4",
    color: "#e85f33",
    duration: 1600,
    name: "用户反馈数",
    value: "",
    percent: "%",
    data: []
  },
  {
    icon: CheckLine,
    bgColor: "#eff8f4",
    color: "#26ce83",
    duration: 1500,
    name: "反馈解决率",
    value: "",
    percent: "",
    data: []
  },
  {
    icon: Smile,
    bgColor: "#f6f4fe",
    color: "#7846e5",
    duration: 100,
    name: "升级包校验通过率",
    value: "",
    percent: "",
    data: []
  }
]);

/** 下载数 */
const barChartData = ref([
  {
    requireData: [],
    questionData: []
  },
  {
    requireData: [],
    questionData: []
  }
]);

/** 用户安装成功概率 */
const progressData = ref([
  {
    week: "周一",
    percentage: 0,
    duration: 110,
    color: "#41b6ff"
  },
  {
    week: "周二",
    percentage: 0,
    duration: 105,
    color: "#41b6ff"
  },
  {
    week: "周三",
    percentage: 0,
    duration: 100,
    color: "#41b6ff"
  },
  {
    week: "周四",
    percentage: 0,
    duration: 95,
    color: "#41b6ff"
  },
  {
    week: "周五",
    percentage: 0,
    duration: 90,
    color: "#26ce83"
  },
  {
    week: "周六",
    percentage: 0,
    duration: 85,
    color: "#26ce83"
  },
  {
    week: "周日",
    percentage: 0,
    duration: 80,
    color: "#26ce83"
  }
]).value.reverse();

/** 数据统计 */
/** table数据：升级包表编号，升级包表名称，升级包表下载数，升级包表反馈数，升级包表反馈解决数 */
const tableData = ref([]);

/** 最新动态 */
const latestNewsData = ref([]);

async function init() {
  await getPackageSummary().then(res => {
    if (res.success) {
      chartData.value.at(0).value = res.data?.value;
      chartData.value.at(0).percent = res.data?.percent;
      chartData.value.at(0).data = res.data?.data;
    } else {
      message("获取升级包数失败", { type: "error" })
    }
  })
  await getFeedbackSummary().then(res => {
    if (res.success) {
      chartData.value.at(1).value = res.data?.value;
      chartData.value.at(1).percent = res.data?.percent;
      chartData.value.at(1).data = res.data?.data;
    } else {
      message("获取用户反馈数失败", { type: "error" })
    }
  })
  await getFeedbackSolved().then(res => {
    if (res.success) {
      chartData.value.at(2).value = res.data?.value;
      chartData.value.at(2).percent = res.data?.percent;
      chartData.value.at(2).data = res.data?.data;
    } else {
      message("获取反馈解决率失败", { type: "error" })
    }
  })
  await getPackagePassRate().then(res => {
    if (res.success) {
      chartData.value.at(3).value = res.data?.value;
      chartData.value.at(3).percent = res.data?.percent;
      chartData.value.at(3).data = res.data?.data;
    } else {
      message("获取升级包校验通过率失败", { type: "error" })
    }
  })
  await getDownloadData().then(res => {
    if (res.success) {
      barChartData.value.at(0).requireData = res.data?.at(0)?.requireData;
      barChartData.value.at(0).questionData = res.data?.at(0)?.questionData;
      barChartData.value.at(1).requireData = res.data?.at(1)?.requireData;
      barChartData.value.at(1).questionData = res.data?.at(1)?.questionData;
    } else {
      message("获取下载数失败", { type: "error" })
    }
  })
  await getInstallData().then(res => {
    if (res.success) {
      progressData.forEach((item, index) => {
        item.percentage = res.data?.at(index).percentage;
      })
    } else {
      message("获取用户安装成功概率失败", { type: "error" })
    }
  })
  await getTableData().then(res => {
    if (res.success) {
      tableData.value = res.data;
    } else {
      message("获取表格数据失败", { type: "error" })
    }
  })
  await getFeedbackDynamic().then(res => {
    if (res.success) {
      latestNewsData.value = res.data;
    } else {
      message("获取最新动态失败", { type: "error" })
    }
  })
}





export { chartData, barChartData, progressData, tableData, latestNewsData,init };
